/*
 * floppyfileentry.c
 *
 * Copyright 2004 Martin Garton
 * Copyright 2004 Richard Drummond
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "util.h"
#include "floppyfileentry.h"
#include "led.h"

static void floppyfileentry_init (FloppyFileEntry *ffe);
static void floppyfileentry_class_init (FloppyFileEntryClass *class);
static void on_eject (GtkWidget *w, FloppyFileEntry *ffe);
static void on_insert (GtkWidget *w, FloppyFileEntry *ffe);

guint floppyfileentry_get_type ()
{
    static guint floppyfileentry_type = 0;

    if (!floppyfileentry_type) {
	GtkTypeInfo floppyfileentry_info = {
	    "FloppyFileEntry",
	    sizeof (FloppyFileEntry),
	    sizeof (FloppyFileEntryClass),
	    (GtkClassInitFunc) floppyfileentry_class_init,
	    (GtkObjectInitFunc) floppyfileentry_init,
	    NULL,
	    NULL,
	    (GtkClassInitFunc) NULL
	};
	floppyfileentry_type = gtk_type_unique (gtk_frame_get_type (), &floppyfileentry_info);
    }
    return floppyfileentry_type;
}

enum {
    DISC_CHANGE_SIGNAL,
    LAST_SIGNAL
};

static gint floppyfileentry_signals[LAST_SIGNAL] = { 0 };

static void floppyfileentry_class_init (FloppyFileEntryClass *class)
{
    gtkutil_add_signals_to_class ((GtkObjectClass *)class,
				   GTK_STRUCT_OFFSET (FloppyFileEntryClass, floppyfileentry),
				   floppyfileentry_signals,
				   "disc-changed",
				   0);

    class->floppyfileentry = NULL;
}

static void floppyfileentry_init (FloppyFileEntry *ffe)
{
    GtkWidget *hbox;
    GtkWidget *ledbox;

    gtk_container_set_border_width (GTK_CONTAINER (ffe), 8);
    gtk_frame_set_label_align (GTK_FRAME (ffe), 0.01, 0.5);

    ffe->led = led_new ();
    ledbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (ledbox), gtk_hbox_new (FALSE, 0));
    gtk_container_add (GTK_CONTAINER (ledbox), ffe->led);
    gtk_container_add (GTK_CONTAINER (ledbox), gtk_hbox_new (FALSE, 0));

    ffe->label = gtk_label_new ("");
    ffe->label_frame = gtk_frame_new (NULL);
    gtk_frame_set_shadow_type (GTK_FRAME (ffe->label_frame), GTK_SHADOW_ETCHED_OUT);
    gtk_container_add (GTK_CONTAINER (ffe->label_frame), ffe->label);

    ffe->insert_button = gtk_button_new_with_label ("Insert");
    ffe->eject_button  = gtk_button_new_with_label ("Eject");

    hbox = gtk_hbox_new (FALSE, 4);
    gtk_box_pack_start (GTK_BOX (hbox), ledbox, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox), ffe->label_frame, TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (hbox), ffe->eject_button, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox), ffe->insert_button, FALSE, FALSE, 2);

    gtk_signal_connect (GTK_OBJECT (ffe->eject_button), "clicked",
			GTK_SIGNAL_FUNC (on_eject),
			ffe);

    gtk_container_add (GTK_CONTAINER (ffe), hbox);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 8);
    gtk_widget_show_all (hbox);

    gtk_signal_connect (GTK_OBJECT (ffe->insert_button), "clicked",
			GTK_SIGNAL_FUNC (on_insert),
			ffe);
}


static void on_eject (GtkWidget *w, FloppyFileEntry *ffe)
{
    if (ffe->filename)
	g_free(ffe->filename);
    ffe->filename = NULL;
    gtk_label_set_text (GTK_LABEL (ffe->label), "");
    gtk_signal_emit_by_name (GTK_OBJECT (ffe), "disc-changed");
}

static void on_filesel_close (FloppyFileEntry *ffe, gpointer data)
{
    gtk_widget_set_sensitive (ffe->insert_button, 1);
    gtk_widget_set_sensitive (ffe->eject_button, 1);

    gtk_widget_destroy (ffe->filesel);
}

static void on_filesel_ok (FloppyFileEntry *ffe, gpointer data)
{
    const gchar *fname = gtk_file_selection_get_filename (GTK_FILE_SELECTION (ffe->filesel));

    if (ffe->filename) {
	g_free (ffe->filename);
	ffe->filename = 0;
    }
    if (ffe->currentdir) {
	g_free (ffe->currentdir);
	ffe->currentdir = 0;
    }

    if (fname && strlen (fname)) {
	struct stat statbuf;

	if (stat (fname, &statbuf) == 0) {
	    if (S_ISDIR (statbuf.st_mode)) {
		/* If user selected a directory make sure we
		 * have a trailing path separator
		 */
	        int len = strlen (fname);
		if (fname[len-1] != '/')
		    ffe->currentdir = g_strconcat (fname, "/", NULL);
		else
		    ffe->currentdir = g_strdup (fname);
	    } else {
		const gchar *p = strrchr (fname, '/');

		if (p)
		    ffe->currentdir = g_strndup (fname, p - fname + 1);
		else
		    ffe->currentdir = g_strdup (fname);

		ffe->filename = g_strdup (fname);
	    }
	}
    }

    gtk_label_set_text (GTK_LABEL (ffe->label), ffe->filename);
    gtk_widget_destroy (ffe->filesel);
    gtk_signal_emit_by_name (GTK_OBJECT(ffe), "disc-changed");
    gtk_widget_set_sensitive (ffe->insert_button, 1);
    gtk_widget_set_sensitive (ffe->eject_button, 1);
}

static void on_insert (GtkWidget *w, FloppyFileEntry *ffe)
{
    gchar *title;

    gtk_widget_set_sensitive (ffe->insert_button, 0);
    gtk_widget_set_sensitive (ffe->eject_button, 0);

    title = g_strconcat ("Select floppy image to insert in drive ", ffe->drivename, NULL);

    ffe->filesel = gtk_file_selection_new (title);

    if (ffe->filesel) {
        gtk_file_selection_set_filename (GTK_FILE_SELECTION (ffe->filesel),
					 ffe->currentdir ? ffe->currentdir : "");

	gtk_signal_connect_object (GTK_OBJECT (ffe->filesel),
				   "destroy",
				   (GtkSignalFunc) on_filesel_close,
				   (gpointer)ffe);
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (ffe->filesel)->ok_button),
				   "clicked",
				   (GtkSignalFunc) on_filesel_ok,
				   (gpointer)ffe);

	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (ffe->filesel)->cancel_button),
				   "clicked",
				   (GtkSignalFunc) on_filesel_close,
				   GTK_OBJECT (ffe));
	gtk_widget_show (ffe->filesel);
    } else {
	gtk_widget_set_sensitive (ffe->insert_button, 1);
	gtk_widget_set_sensitive (ffe->eject_button, 1);
    }

    if (title)
	g_free (title);
}

GtkWidget *floppyfileentry_new (void)
{
    FloppyFileEntry *w = FLOPPYFILEENTRY (gtk_type_new (floppyfileentry_get_type ()));

    return GTK_WIDGET (w);
}

void floppyfileentry_set_drivename (FloppyFileEntry *ffe, const gchar *drivename)
{
    if (ffe->drivename)
	g_free (ffe->drivename);
    ffe->drivename = g_strdup (drivename);
}

void floppyfileentry_set_currentdir (FloppyFileEntry *ffe, const gchar *pathname)
{
    if (ffe->currentdir)
        g_free (ffe->currentdir);
    ffe->currentdir = g_strdup ((gchar *)pathname);
}

void floppyfileentry_set_filename (FloppyFileEntry *ffe, const gchar *filename)
{
    gtk_label_set_text (GTK_LABEL (ffe->label), filename);
}

#if GTK_MAJOR_VERSION > 2
G_CONST_RETURN
#endif
gchar *floppyfileentry_get_filename (FloppyFileEntry *ffe)
{
    return ffe->filename;
}

void floppyfileentry_set_label (FloppyFileEntry *ffe, const gchar *label)
{
    gtk_frame_set_label (GTK_FRAME (ffe), label);
}

void floppyfileentry_set_led (FloppyFileEntry *ffe, gboolean state)
{
    GdkColor color;

    if (state)
	color = (GdkColor){0, 0x0000, 0xFFFF, 0x0000};
    else
	color = (GdkColor){0, 0x0000, 0x0000, 0x0000};

    led_set_color (LED (ffe->led), color);
}

void floppyfileentry_do_dialog (FloppyFileEntry *ffe)
{
    on_insert (NULL, ffe);
}
